// Codesign Challenge ECE 4530 Fall 2008
// P. Schaumont
//
// This reference implementation counts the number of bits
// in a large array.
//
// The objective of the Codesign Challenge is to speedup the
// counting process as much as possible.
//
#include "xparameters.h"
#include "xtmrctr.h"

#define MAXDATA 16384

// the array with reference data
unsigned tdata[MAXDATA];

//----------------------------------------
// Bit counting algorithm
//

static int n;
static int idx;
static int endofdata;
void initbit() {
  n = 0;
  idx = 0;
  endofdata = 0;
}

unsigned nextbit() {
  unsigned bit;
  bit = (tdata[n] >> (31 - idx)) & 1;
  idx++;
  if (idx == 32) {
    idx = 0;
    n++;
    if (n == MAXDATA)
      endofdata = 1;
  }
  return bit;
}

unsigned countbits() {
  unsigned len;
  len = 0;
  initbit();
  while (endofdata == 0) 
    if (nextbit())
      len++;
  return len;
}

//----------------------------------------
// fill array with pseudorandom data (approximately 50% '1')
void initialize() {
  unsigned int lfsr = 1;
  unsigned int p, r; 
  for (p=0; p<MAXDATA; ++p) {
    for (r=0; r<32; r++) 
      lfsr = (lfsr >> 1) ^ (-(lfsr & 1u) & 0xd0000001u); 
    tdata[p] = lfsr;    
  }
}

void clear() {
  unsigned p;
  for (p=0; p<MAXDATA; ++p) 
    tdata[p] = 0;
}

// insert a trace of len '1' bits at index idx
void traceone(unsigned idx, unsigned len) {
  unsigned i;
  if ((idx + len) >= MAXDATA)
    return;
  for (i=idx; i<idx+len; ++i)
    tdata[i] = 0xffffffff;
}

//----------------------------------------
// count bits

int main() {
  unsigned c1, c2;
  unsigned t1, t2;
  XTmrCtr T;
  
  XTmrCtr_Initialize(&T, XPAR_XPS_TIMER_0_DEVICE_ID);
  initialize();

  // REFERENCE IMPLEMENTATION
  XTmrCtr_Start(&T, 0);
  c1 = countbits();
  XTmrCtr_Stop(&T, 0);
  t1 = XTmrCtr_GetValue(&T, 0);

  xil_printf("Bit Count %8x cycles %8d\n", c1, t1);

  return 0;
}

